//    hskymon  from HDS OPE file Editor
//          New SkyMonitor for Subaru Gen2
//      post_eso.h  --- POST body for FCDB access to ESO archive
//   
//                                           2017.11.15  A.Tajitsu

typedef struct _PARAMesoinst PARAMesoinst;
struct _PARAMesoinst{
  gchar *name;
  gchar *prm;
  gchar *add;
};

#define NUM_ESO_IMAGE 17
#define NUM_ESO_SPEC 21
#define NUM_ESO_VLTI 5
#define NUM_ESO_POLA 7
#define NUM_ESO_CORO 4
#define NUM_ESO_OTHER 6
#define NUM_ESO_SAM 2

// ESO Obs mode
enum
{
  TRDB_ESO_MODE_IMAGE,
  TRDB_ESO_MODE_SPEC,
  TRDB_ESO_MODE_VLTI,
  TRDB_ESO_MODE_POLA,
  TRDB_ESO_MODE_CORO,
  TRDB_ESO_MODE_OTHER,
  TRDB_ESO_MODE_SAM
};

static const PARAMesoinst eso_image[NUM_ESO_IMAGE] = {
  {"EFOSC2/Lasilla",  
   "EFOSC;\'IMA%\'", 
   "(ins_id like \'EFOSC%\' AND (dp_tech like \'IMA%\'))"},
  {"EMMI/Lasilla",  
   "EMMI;\'IMA%\'", 
   "(ins_id like \'EMMI%\' AND (dp_tech like \'IMA%\'))"},
  {"FORS1/VLT",  
   "FORS1;\'IMA%\'", 
   "(ins_id like \'FORS1%\' AND (dp_tech like \'IMA%\'))"},
  {"FORS2/VLT",  
   "FORS2;\'IMA%\'", 
   "(ins_id like \'FORS2%\' AND (dp_tech like \'IMA%\'))"},
  {"HAWKI/VLT",  
   "HAWKI;\'IMA%\'", 
   "(ins_id like \'HAWKI%\' AND (dp_tech like \'IMA%\'))"},
  {"GROND/LaSilla",  
   "GROND", 
   "(ins_id like \'GROND%\')"},
  {"ISAAC/VLT",  
   "ISAAC;\'IMA%\'", 
   "(ins_id like \'ISAAC%\' AND (dp_tech like \'IMA%\'))"},
  {"NACO/VLT",  
   "NAOS+CONICA;\'IMA%\'", 
   "(ins_id like \'NAOS+CONICA%\' AND (dp_tech like \'IMA%\'))"},
  {"OMEGACAM/VST",  
   "OMEGACAM", 
   "(ins_id like \'OMEGACAM%\')"},
  {"SOFI/LaSilla",  
   "SOFI;\'IMA%\'", 
   "(ins_id like \'SOFI%\' AND (dp_tech like \'IMA%\'))"},
  {"SPHERE/VLT",  
   "SPHERE;\'IMA%\'",
   "(ins_id like \'SPHERE%\' AND (dp_tech like \'IMA%\'))"},
  {"SUSI/2/LaSilla",  
   "SUSI",
   "(ins_id like \'SUSI%\')"},
  {"TIMMI2/LaSilla",  
   "TIMMI;\'IMA%\'",
   "(ins_id like \'TIMMI%\' AND (dp_tech like \'IMA%\'))"},
  {"VIMOS/VLT",  
   "VIMOS;\'IMA%\'",
   "(ins_id like \'VIMOS%\' AND (dp_tech like \'IMA%\'))"},
  {"VIRCAM/VISTA",  
   "VIRCAM",
   "(ins_id like \'VIRCAM%\')"},
  {"VISIR/VLT",  
   "VISIR;\'IMA%\'",
   "(ins_id like \'VISIR%\' AND (dp_tech like \'IMA%\'))"},
  {"WFI/LaSilla",  
   "WFI",
   "(ins_id like \'WFI%\')"}};

static const PARAMesoinst eso_spec[NUM_ESO_SPEC] = {
  {"CES/LaSilla",
   "CES",
   "(ins_id like \'CES%\')"},
  {"CRIRES/VLT",  
   "CRIRE;\'SPECTRUM%\'",
   "(ins_id like \'CRIRE%\' AND ((dp_tech like \'SPECTRUM%\') OR (dp_cat != \'SCIENCE\')))"},
  {"EFOSC2/LaSilla",  
   "EFOSC;\'SPECTRUM%\'",
   "(ins_id like \'EFOSC%\' AND ((dp_tech like \'SPECTRUM%\') OR (dp_cat != \'SCIENCE\')))"},
  {"EMMI/LaSilla",  
   "EMMI;\'SPECTRUM%\',\'ECHELLE%\',\'MOS%\'",
   "(ins_id like \'EMMI%\' AND ((dp_tech like \'SPECTRUM%\' OR dp_tech like \'ECHELLE%\' OR dp_tech like \'MOS%\') OR (dp_cat != \'SCIENCE\')))"},
  {"FEROS/LaSilla",  
   "FEROS",
   "(ins_id like \'FEROS%\')"},
  {"FORS1/VLT",  
   "FORS1;\'SPECTRUM%\',\'MOS%\',\'IMAGE_SPECTRUM%\'",
   "(ins_id like \'FORS1%\' AND ((dp_tech like \'SPECTRUM%\' OR dp_tech like \'MOS%\' OR dp_tech like \'IMAGE_SPECTRUM%\') OR (dp_cat != \'SCIENCE\')))"},
  {"FORS2/VLT",  
   "FORS2;\'SPECTRUM%\',\'ECHELLE%\',\'MOS%\',\'MXU%\',\'HIT%\',\'IMAGE_SPECTRUM%\'",
   "(ins_id like \'FORS2%\' AND ((dp_tech like \'SPECTRUM%\' OR dp_tech like \'ECHELLE%\' OR dp_tech like \'MOS%\' OR dp_tech like \'MXU%\' OR dp_tech like \'HIT%\' OR dp_tech like \'IMAGE_SPECTRUM%\') OR (dp_cat != \'SCIENCE\')))"},
  {"GIRAFFE/VLT",  
   "GIRAF",
   "(ins_id like \'GIRAF%\')"},
  {"HARPS/LaSilla",  
   "HARPS",
   "(ins_id like \'HARPS%\')"},
  {"ISAAC/VLT",  
   "ISAAC;\'SPECTRUM%\'",
   "(ins_id like \'ISAAC%\' AND ((dp_tech like \'SPECTRUM%\') OR (dp_cat != \'SCIENCE\')))"},
  {"KMOS/VLT",  
   "KMOS",
   "(ins_id like \'KMOS%\')"},
  {"MUSE/VLT",  
   "MUSE",
   "(ins_id like \'MUSE%\')"},
  {"NACO/VLT",  
   "NAOS+CONICA;\'SPECTRUM%\'",
   "(ins_id like \'NAOS+CONICA%\' AND ((dp_tech like \'SPECTRUM%\') OR (dp_cat != \'SCIENCE\')))"},
  {"SINFONI/VLT",  
   "SINFO",
   "(ins_id like \'SINFO%\')"},
  {"SOFI/LaSilla",  
   "SOFI;\'SPECTRUM%\'",
   "(ins_id like \'SOFI%\' AND ((dp_tech like \'SPECTRUM%\') OR (dp_cat != \'SCIENCE\')))"},
  {"SPHERE/VLT",  
   "SPHERE;\'IFU\',\'SPECTRUM%\'",
   "(ins_id like \'SPHERE%\' AND ((dp_tech like \'IFU\' OR dp_tech like \'SPECTRUM%\') OR (dp_cat != \'SCIENCE\')))"},
  {"TIMMI2/LaSilla",  
   "TIMMI;\'SPECTRUM%\'",
   "(ins_id like \'TIMMI%\' AND ((dp_tech like \'SPECTRUM%\') OR (dp_cat != \'SCIENCE\')))"},
  {"UVES/VLT",  
   "UVES",
   "(ins_id like \'UVES%\')"},
  {"VIMOS/VLT",  
   "VIMOS;\'IFU%\',\'MOS%\'",
   "(ins_id like \'VIMOS%\' AND ((dp_tech like \'IFU%\' OR dp_tech like \'MOS%\') OR (dp_cat != \'SCIENCE\')))"},
  {"VISIR/VLT",  
   "VISIR;\'SPECTRUM%\',\'ECHELLE%\'",
   "(ins_id like \'VISIR%\' AND ((dp_tech like \'SPECTRUM%\' OR dp_tech like \'ECHELLE%\') OR (dp_cat != \'SCIENCE\')))"},
  {"XSHOOTER/VLT",  
   "SHOOT",
   "(ins_id like \'SHOOT%\') or (ins_id like \'XSHOOTER%\')"}};

static const PARAMesoinst eso_vlti[NUM_ESO_VLTI] = {
  {"AMBER/VLT",  
   "AMBER",
   "(ins_id like \'AMBER%\')"},
  {"GRAVITY/VLT",  
   "GRAVITY",
   "(ins_id like \'GRAVITY%\')"},
  {"MIDI/VLT",  
   "MIDI",
   "(ins_id like \'MIDI%\')"},
  {"PIONIER/VLT",  
   "PIONIER",
   "(ins_id like \'PIONIER%\')"},
  {"VINCI/VLT",  
   "VINCI",
   "(ins_id like \'VINCI%\')"}};

static const PARAMesoinst eso_pola[NUM_ESO_POLA] = {
  {"EFOSC2/LaSilla",  
   "EFOSC;\'POLARIM%\'",
   "(ins_id like \'EFOSC%\' AND (dp_tech like \'POLARIM%\'))"},
  {"FORS1/VLT",  
   "FORS1;\'POLARIM%\'",
   "(ins_id like \'FORS1%\' AND (dp_tech like \'POLARIM%\'))"},
  {"FORS2/VLT",  
   "FORS2;\'POLARIM%\'",
   "(ins_id like \'FORS2%\' AND (dp_tech like \'POLARIM%\'))"},
  {"ISAAC/VLT",  
   "ISAAC;\'POLARIM%\'",
   "(ins_id like \'ISAAC%\' AND (dp_tech like \'POLARIM%\'))"},
  {"NACO/VLT",  
   "NAOS+CONICA;\'POLARIM%\'",
   "(ins_id like \'NAOS+CONICA%\' AND (dp_tech like \'POLARIM%\'))"},
  {"SOFI/LaSilla",  
   "SOFI;\'POLARIM%\'",
   "(ins_id like \'SOFI%\' AND (dp_tech like \'POLARIM%\'))"},
  {"SPHERE/VLT",  
   "SPHERE;\'POLARIM%\'",
   "(ins_id like \'SPHERE%\' AND (dp_tech like \'POLARIM%\'))"}};

static const PARAMesoinst eso_coro[NUM_ESO_CORO] = {
  {"EFOSC2/LaSilla",  
   "EFOSC;\'%CORO%\'",
   "(ins_id like \'EFOSC%\' AND (dp_tech like \'%CORO%\'))"},
  {"NACO/VLT",  
   "NAOS+CONICA;\'%CORO%\'",
   "(ins_id like \'NAOS+CONICA%\' AND (dp_tech like \'%CORO%\'))"},
  {"SPHERE/VLT",  
   "SPHERE;\'%CORO%\'",
   "(ins_id like \'SPHERE%\' AND (dp_tech like \'%CORO%\'))"},
  {"VISIR/VLT",  
   "VISIR;\'%CORO%\'",
   "(ins_id like \'VISIR%\' AND (dp_tech like \'%CORO%\'))"}};

static const PARAMesoinst eso_other[NUM_ESO_OTHER] = {
  {"BOL/APEX",  
   "APEXBOL",
   "(ins_id like \'APEXBOL%\')"},
  {"HET/APEX",  
   "APEXHET",
   "(ins_id like \'APEXHET%\')"},
  {"LGSF/VLT",  
   "LGSF",
   "(ins_id like \'LGSF%\')"},
  {"MAD/VLT",  
   "MAD",
   "(ins_id like \'MAD%\')"},
  {"MASCOT/Paranal",  
   "MASCOT",
   "(ins_id like \'MASCOT%\')"},
  {"WFCAM/UKIRT",  
   "WFCAM",
   "(ins_id like \'WFCAM%\')"}};

static const PARAMesoinst eso_sam[NUM_ESO_SAM] = {
  {"NACO/VLT",  
   "NAOS+CONICA;\'%SAM%\'",
   "(ins_id like \'NAOS+CONICA%\' AND (dp_tech like \'%SAM%\'))"},
  {"VISIR/VLT",  
   "VISIR;\'SAM%\'",
   "(ins_id like \'VISIR%\' AND (dp_tech like \'SAM%\'))"}};


static const PARAMpost eso_post[] = {
  {POST_INPUT, "wdbo",              NULL}, //"votable/display" or "html/display"
  {POST_CONST, "max_rows_returned", "5000"},
  {POST_NULL,  "instrument",        NULL},
  {POST_CONST, "tab_object",        "on"},
  {POST_NULL,  "target",            NULL},
  {POST_CONST, "resolver",          "simbad"},
  {POST_INPUT, "ra",                NULL},
  {POST_INPUT, "dec",               NULL},
  {POST_INPUT, "box",               NULL},  // 00 10 00
  {POST_CONST, "deg_or_hour",       "degrees"},
  {POST_CONST, "tab_target_coord",  "on"},
  {POST_CONST, "format",            "DecimDeg"},
  {POST_NULL,  "night",             NULL},
  {POST_INPUT, "stime",             NULL},
  {POST_CONST, "starttime",         "12"},
  {POST_INPUT, "etime",             NULL},
  {POST_CONST, "endtime",           "12"},
  {POST_CONST, "tab_prog_id",       "on"},
  {POST_NULL,  "prog_id",           NULL},
  {POST_NULL,  "gto",               NULL},
  {POST_NULL,  "pi_coi",            NULL},
  {POST_NULL,  "obs_mode",          NULL},
  {POST_NULL,  "title",             NULL},
  {POST_INST1, "image[]",           NULL},
  {POST_INST2, "spectrum[]",        NULL},
  {POST_INST3, "vlti[]",            NULL},
  {POST_INST4, "polarim[]",         NULL},
  {POST_INST5, "corono[]",          NULL},
  {POST_INST6, "other[]",           NULL},
  {POST_INST7, "sam[]",             NULL},
  {POST_CONST, "tab_dp_cat",        "on"},
  {POST_CONST, "dp_cat",            "SCIENCE"},
  {POST_CONST, "tab_dp_type",       "on"},
  {POST_CONST, "dp_type",           "OBJECT"},
  {POST_NULL,  "dp_type_user",      NULL},
  {POST_CONST, "tab_dp_tech",       "on"},
  {POST_NULL,  "dp_tech",           NULL},
  {POST_NULL,  "dp_tech_user",      NULL},
  {POST_CONST, "tab_dp_id",         "on"},
  {POST_NULL,  "dp_id",             NULL},
  {POST_NULL,  "origfile",          NULL},
  {POST_CONST, "tab_rel_date",      "on"},
  {POST_NULL,  "rel_date",          NULL},
  {POST_NULL,  "obs_name",          NULL},
  {POST_NULL,  "ob_id",             NULL},
  {POST_CONST, "tab_tpl_start",     "on"},
  {POST_NULL,  "tpl_start",         NULL},
  {POST_CONST, "tab_tpl_id",        "on"},
  {POST_NULL,  "tpl_id",            NULL},
  {POST_CONST, "tab_exptime",       "on"},
  {POST_NULL,  "exptime",           NULL},
  {POST_CONST, "tab_filter_path",   "on"},
  {POST_NULL,  "filter_path",       NULL},
  {POST_NULL,  "gris_path",         NULL},
  {POST_NULL,  "grat_path",         NULL},
  {POST_NULL,  "slit_path",         NULL},
  {POST_CONST, "tab_instrument",   "on"},
  {POST_CONST, "tab_instrument",   "on"},
  {POST_ADD,   "add",              NULL},
  {POST_CONST, "tab_tel_airm_start",   "on"},
  {POST_CONST, "tab_stat_instrument",  "on"},
  {POST_CONST, "tab_ambient",          "on"},
  {POST_CONST, "tab_stat_exptime",     "on"},
  {POST_CONST, "tab_HDR",              "on"},
  {POST_CONST, "tab_mjd_obs",          "on"},
  {POST_CONST, "aladin_colour",        "aladin_instrument"},
  {POST_CONST, "tab_stat_plot",        "on"},
  {POST_NULL,  "order",                NULL},
  {POST_NULL,  NULL, NULL}};
